@@ -1,10 +1,11 @@
+# SPDX-License-Identifier: MIT
+
 """
 Tests for `attr._funcs`.
 """
 
-from __future__ import absolute_import, division, print_function
 
-from collections import Mapping, OrderedDict, Sequence
+from collections import OrderedDict
 
 import pytest
 
@@ -14,7 +15,7 @@ from hypothesis import strategies as st
 import attr
 
 from attr import asdict, assoc, astuple, evolve, fields, has
-from attr._compat import TYPE
+from attr._compat import Mapping, Sequence
 from attr.exceptions import AttrsAttributeNotFoundError
 from attr.validators import instance_of
 
@@ -25,7 +26,22 @@ MAPPING_TYPES = (dict, OrderedDict)
 SEQUENCE_TYPES = (list, tuple)
 
 
-class TestAsDict(object):
+@pytest.fixture(scope="session", name="C")
+def _C():
+    """
+    Return a simple but fully featured attrs class with an x and a y attribute.
+    """
+    import attr
+
+    @attr.s
+    class C:
+        x = attr.ib()
+        y = attr.ib()
+
+    return C
+
+
+class TestAsDict:
     """
     Tests for `asdict`.
     """
@@ -134,12 +150,26 @@ class TestAsDict(object):
             retain_collection_types=True,
         )
 
+    @given(set_type=st.sampled_from((set, frozenset)))
+    def test_sets_no_retain(self, C, set_type):
+        """
+        Set types are converted to lists if retain_collection_types=False.
+        """
+        d = asdict(
+            C(1, set_type((1, 2, 3))),
+            retain_collection_types=False,
+            recurse=True,
+        )
+
+        assert {"x": 1, "y": [1, 2, 3]} == d
+
     @given(st.sampled_from(MAPPING_TYPES))
     def test_dicts(self, C, dict_factory):
         """
         If recurse is True, also recurse into dicts.
         """
         res = asdict(C(1, {"a": C(4, 5)}), dict_factory=dict_factory)
+
         assert {"x": 1, "y": {"a": {"x": 4, "y": 5}}} == res
         assert isinstance(res, dict_factory)
 
@@ -163,15 +193,46 @@ class TestAsDict(object):
     @given(simple_classes())
     def test_asdict_preserve_order(self, cls):
         """
-        Field order should be preserved when dumping to OrderedDicts.
+        Field order should be preserved when dumping to an ordered_dict.
         """
         instance = cls()
-        dict_instance = asdict(instance, dict_factory=OrderedDict)
+        dict_instance = asdict(instance, dict_factory=dict)
 
         assert [a.name for a in fields(cls)] == list(dict_instance.keys())
 
+    def test_retain_keys_are_tuples(self):
+        """
+        retain_collect_types also retains keys.
+        """
 
-class TestAsTuple(object):
+        @attr.s
+        class A:
+            a = attr.ib()
+
+        instance = A({(1,): 1})
+
+        assert {"a": {(1,): 1}} == attr.asdict(
+            instance, retain_collection_types=True
+        )
+
+    def test_tuple_keys(self):
+        """
+        If a key is collection type, retain_collection_types is False,
+         the key is serialized as a tuple.
+
+        See #646
+        """
+
+        @attr.s
+        class A:
+            a = attr.ib()
+
+        instance = A({(1,): 1})
+
+        assert {"a": {(1,): 1}} == attr.asdict(instance)
+
+
+class TestAsTuple:
     """
     Tests for `astuple`.
     """
@@ -233,14 +294,14 @@ class TestAsTuple(object):
                 elif isinstance(field_val, (list, tuple)):
                     # This field holds a sequence of something.
                     expected_type = type(obj_tuple[index])
-                    assert type(field_val) is expected_type  # noqa: E721
+                    assert type(field_val) is expected_type
                     for obj_e, obj_tuple_e in zip(field_val, obj_tuple[index]):
                         if has(obj_e.__class__):
                             assert_proper_col_class(obj_e, obj_tuple_e)
                 elif isinstance(field_val, dict):
                     orig = field_val
                     tupled = obj_tuple[index]
-                    assert type(orig) is type(tupled)  # noqa: E721
+                    assert type(orig) is type(tupled)
                     for obj_e, obj_tuple_e in zip(
                         orig.items(), tupled.items()
                     ):
@@ -315,8 +376,21 @@ class TestAsTuple(object):
 
         assert instance == roundtrip_instance
 
+    @given(set_type=st.sampled_from((set, frozenset)))
+    def test_sets_no_retain(self, C, set_type):
+        """
+        Set types are converted to lists if retain_collection_types=False.
+        """
+        d = astuple(
+            C(1, set_type((1, 2, 3))),
+            retain_collection_types=False,
+            recurse=True,
+        )
 
-class TestHas(object):
+        assert (1, [1, 2, 3]) == d
+
+
+class TestHas:
     """
     Tests for `has`.
     """
@@ -333,7 +407,7 @@ class TestHas(object):
         """
 
         @attr.s
-        class D(object):
+        class D:
             pass
 
         assert has(D)
@@ -345,7 +419,7 @@ class TestHas(object):
         assert not has(object)
 
 
-class TestAssoc(object):
+class TestAssoc:
     """
     Tests for `assoc`.
     """
@@ -357,7 +431,7 @@ class TestAssoc(object):
         """
 
         @attr.s(slots=slots, frozen=frozen)
-        class C(object):
+        class C:
             pass
 
         i1 = C()
@@ -409,9 +483,7 @@ class TestAssoc(object):
         ) as e, pytest.deprecated_call():
             assoc(C(), aaaa=2)
 
-        assert (
-            "aaaa is not an attrs attribute on {cls!r}.".format(cls=C),
-        ) == e.value.args
+        assert (f"aaaa is not an attrs attribute on {C!r}.",) == e.value.args
 
     def test_frozen(self):
         """
@@ -419,7 +491,7 @@ class TestAssoc(object):
         """
 
         @attr.s(frozen=True)
-        class C(object):
+        class C:
             x = attr.ib()
             y = attr.ib()
 
@@ -432,7 +504,7 @@ class TestAssoc(object):
         """
 
         @attr.s
-        class C(object):
+        class C:
             x = attr.ib()
 
         with pytest.warns(DeprecationWarning) as wi:
@@ -441,7 +513,7 @@ class TestAssoc(object):
         assert __file__ == wi.list[0].filename
 
 
-class TestEvolve(object):
+class TestEvolve:
     """
     Tests for `evolve`.
     """
@@ -453,7 +525,7 @@ class TestEvolve(object):
         """
 
         @attr.s(slots=slots, frozen=frozen)
-        class C(object):
+        class C:
             pass
 
         i1 = C()
@@ -502,8 +574,15 @@ class TestEvolve(object):
         # No generated class will have a four letter attribute.
         with pytest.raises(TypeError) as e:
             evolve(C(), aaaa=2)
-        expected = "__init__() got an unexpected keyword argument 'aaaa'"
-        assert (expected,) == e.value.args
+
+        if hasattr(C, "__attrs_init__"):
+            expected = (
+                "__attrs_init__() got an unexpected keyword argument 'aaaa'"
+            )
+        else:
+            expected = "__init__() got an unexpected keyword argument 'aaaa'"
+
+        assert e.value.args[0].endswith(expected)
 
     def test_validator_failure(self):
         """
@@ -511,14 +590,14 @@ class TestEvolve(object):
         """
 
         @attr.s
-        class C(object):
+        class C:
             a = attr.ib(validator=instance_of(int))
 
         with pytest.raises(TypeError) as e:
             evolve(C(a=1), a="some string")
         m = e.value.args[0]
 
-        assert m.startswith("'a' must be <{type} 'int'>".format(type=TYPE))
+        assert m.startswith("'a' must be <class 'int'>")
 
     def test_private(self):
         """
@@ -526,7 +605,7 @@ class TestEvolve(object):
         """
 
         @attr.s
-        class C(object):
+        class C:
             _a = attr.ib()
 
         assert evolve(C(1), a=2)._a == 2
@@ -543,8 +622,56 @@ class TestEvolve(object):
         """
 
         @attr.s
-        class C(object):
+        class C:
             a = attr.ib()
             b = attr.ib(init=False, default=0)
 
         assert evolve(C(1), a=2).a == 2
+
+    def test_regression_attrs_classes(self):
+        """
+        evolve() can evolve fields that are instances of attrs classes.
+
+        Regression test for #804
+        """
+
+        @attr.s
+        class Cls1:
+            param1 = attr.ib()
+
+        @attr.s
+        class Cls2:
+            param2 = attr.ib()
+
+        obj2a = Cls2(param2="a")
+        obj2b = Cls2(param2="b")
+
+        obj1a = Cls1(param1=obj2a)
+
+        assert Cls1(param1=Cls2(param2="b")) == attr.evolve(
+            obj1a, param1=obj2b
+        )
+
+    def test_dicts(self):
+        """
+        evolve() can replace an attrs class instance with a dict.
+
+        See #806
+        """
+
+        @attr.s
+        class Cls1:
+            param1 = attr.ib()
+
+        @attr.s
+        class Cls2:
+            param2 = attr.ib()
+
+        obj2a = Cls2(param2="a")
+        obj2b = {"foo": 42, "param2": 42}
+
+        obj1a = Cls1(param1=obj2a)
+
+        assert Cls1({"foo": 42, "param2": 42}) == attr.evolve(
+            obj1a, param1=obj2b
+        )
