@@ -1,19 +1,16 @@
+# SPDX-License-Identifier: MIT
+
 """
 Tests for `__init_subclass__` related tests.
-
-Python 3.6+ only.
 """
 
-import pytest
-
 import attr
 
 
-@pytest.mark.parametrize("slots", [True, False])
 def test_init_subclass_vanilla(slots):
     """
     `super().__init_subclass__` can be used if the subclass is not an attrs
-    class both with dict and slots classes.
+    class both with dict and slotted classes.
     """
 
     @attr.s(slots=slots)
